@@ -46,6 +46,9 @@ typedef struct {
 static const char pg_default_encoding[] = "unicode_escape";
 static const char pg_default_errors[] = "backslashreplace";
 
+#define PATHLIB "pathlib"
+#define PUREPATH "PurePath"
+
 #if IS_SDLv1
 static int
 _pg_rw_seek(SDL_RWops *, int, int);
@@ -160,15 +163,60 @@ fetch_object_methods(pgRWHelper *helper, PyObject *obj)
     return 0;
 }
 
+/* This function is meant to decode a pathlib object into its str/bytes representation.
+ * It is based on PyOS_FSPath, and defines this function on python 3.4, 3.5 */
+static PyObject *
+_trydecode_pathlibobj(PyObject *obj)
+{
+#if PY_VERSION_HEX >= 0x03060000
+    PyObject *ret = PyOS_FSPath(obj);
+    if (!ret) {
+        /* A valid object was not passed. But we do not consider it an error */
+        PyErr_Clear();
+        Py_INCREF(obj);
+        return obj;
+    }
+    return ret;
+#elif PY_VERSION_HEX >= 0x03040000
+    /* Custom implementation for back-compat */
+    int ret;
+    PyObject *pathlib, *purepath;
+
+    pathlib = PyImport_ImportModule(PATHLIB);
+    if (!pathlib)
+        return NULL;
+
+    purepath = PyObject_GetAttrString(pathlib, PUREPATH);
+    if (!purepath) {
+        Py_DECREF(pathlib);
+        return NULL;
+    }
+
+    ret = PyObject_IsInstance(obj, purepath);
+
+    Py_DECREF(pathlib);
+    Py_DECREF(purepath);
+
+    if (ret == 1)
+        return PyObject_Str(obj);
+    else if (ret == 0) {
+        Py_INCREF(obj);
+        return obj;
+    }
+    else
+        return NULL;
+#else
+    /* Pathlib module does not exist, just incref and return */
+    Py_INCREF(obj);
+    return obj;
+#endif
+}
+
 static PyObject *
 pg_EncodeString(PyObject *obj, const char *encoding, const char *errors,
                 PyObject *eclass)
 {
-    PyObject *oencoded;
-    PyObject *exc_type;
-    PyObject *exc_value;
-    PyObject *exc_trace;
-    PyObject *str;
+    PyObject *oencoded, *exc_type, *exc_value, *exc_trace, *str, *ret;
 
     if (obj == NULL) {
         /* Assume an error was raise; forward it */
@@ -180,8 +228,15 @@ pg_EncodeString(PyObject *obj, const char *encoding, const char *errors,
     if (errors == NULL) {
         errors = pg_default_errors;
     }
-    if (PyUnicode_Check(obj)) {
-        oencoded = PyUnicode_AsEncodedString(obj, encoding, errors);
+    
+    ret = _trydecode_pathlibobj(obj);
+    if (!ret)
+        return NULL;
+    
+    if (PyUnicode_Check(ret)) {
+        oencoded = PyUnicode_AsEncodedString(ret, encoding, errors);
+        Py_DECREF(ret);
+
         if (oencoded != NULL) {
             return oencoded;
         }
@@ -215,12 +270,14 @@ pg_EncodeString(PyObject *obj, const char *encoding, const char *errors,
                          " unexpected encoding error");
         }
         PyErr_Clear();
+        Py_RETURN_NONE;
     }
-    else if (Bytes_Check(obj)) {
-        Py_INCREF(obj);
-        return obj;
+    
+    if (Bytes_Check(ret)) {
+        return ret;
     }
 
+    Py_DECREF(ret);
     Py_RETURN_NONE;
 }
 
@@ -461,9 +518,14 @@ pgRWops_FromFileObject(PyObject *obj)
     rw->write = _pg_rw_write;
     rw->close = _pg_rw_close;
 
+/* https://docs.python.org/3/c-api/init.html#c.PyEval_InitThreads */
+/* ^ in Python >= 3.7, we don't have to call this function, and in 3.11 
+ * it will be removed */
+#if PY_VERSION_HEX < 0x03070000
 #ifdef WITH_THREAD
     PyEval_InitThreads();
 #endif /* WITH_THREAD */
+#endif
 
     return rw;
 }
@@ -471,6 +533,7 @@ pgRWops_FromFileObject(PyObject *obj)
 static int
 pgRWops_ReleaseObject(SDL_RWops *context)
 {
+    int ret = 0;
     if (pgRWops_IsFileObject(context)) {
 #ifdef WITH_THREAD
         PyGILState_STATE state = PyGILState_Ensure();
@@ -492,11 +555,10 @@ pgRWops_ReleaseObject(SDL_RWops *context)
             SDL_FreeRW(context);
         }
         else {
-            int ret;
-            if ((ret = SDL_RWclose(context)) < 0) {
+            ret = SDL_RWclose(context);
+            if (ret < 0) {
                 PyErr_SetString(PyExc_IOError, SDL_GetError());
                 Py_DECREF(fileobj);
-                return ret;
             }
         }
 
@@ -505,13 +567,11 @@ pgRWops_ReleaseObject(SDL_RWops *context)
 #endif /* WITH_THREAD */
     }
     else {
-        int ret;
-        if ((ret = SDL_RWclose(context)) < 0) {
+        ret = SDL_RWclose(context);
+        if (ret < 0)
             PyErr_SetString(PyExc_IOError, SDL_GetError());
-            return ret;
-        }
     }
-    return 0;
+    return ret;
 }
 
 #if IS_SDLv1
@@ -607,7 +667,7 @@ _pg_rw_read(SDL_RWops *context, void *ptr, size_t size, size_t maxnum)
 {
     pgRWHelper *helper = (pgRWHelper *)context->hidden.unknown.data1;
     PyObject *result;
-    ssize_t retval;
+    Py_ssize_t retval;
 #endif /* IS_SDLv2 */
 #ifdef WITH_THREAD
     PyGILState_STATE state;
